@@ -1,9 +1,9 @@
 /*
  * ebiso.h
  * 
- * Version:       0.1.1
+ * Version:       0.1.2
  * 
- * Release date:  20.09.2015
+ * Release date:  20.10.2015
  * 
  * Copyright 2015 Vladimir (sodoma) Gozora <c@gozora.sk>
  * 
@@ -33,7 +33,7 @@
 #include <getopt.h>
 
 #define PROGNAME "ebiso"
-#define VERSION "0.1.1"
+#define VERSION "0.1.2"
 
 #ifdef DEBUG
 static void disp_level(struct file_list_t *list_to_display, int level);
@@ -74,7 +74,7 @@ extern int et_boot_catalog(struct ISO_data_t LBA_data);
 int option_on_off(uint32_t option2check, enum opt_l option);
 
 static int set_option(uint32_t *opt2set, enum opt_l option);
-static int check_availability(char *filename, enum check_type_l type, enum check_mode_l mode);
+static int check_availability(char *filename, enum check_type_l type, enum check_mode_l mode, struct file_list_t *file_list);
 static void help_msg(enum msg_l id);
 static void err_msg(enum errors_l error);
 static uint32_t get_path_table_offset(struct file_list_t *file_list);
